/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.Alida.dataio.provider.xmlbeans;

import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerXmlbeans;
import de.unihalle.informatik.Alida.dataio.provider.ALDDataIOXmlbeans;
import de.unihalle.informatik.Alida.dataio.provider.xmlbeans.ALDStandardizedDataIOXmlbeans;
import de.unihalle.informatik.Alida.dataio.provider.helpers.ALDParametrizedClassDataIOHelper;
import de.unihalle.informatik.Alida.dataio.provider.helpers.ALDParametrizedClassDummy;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOManagerException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException.ALDDataIOProviderExceptionType;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.helpers.ALDParser;
import de.unihalle.informatik.Alida.operator.ALDOpParameterDescriptor;
import de.unihalle.informatik.Alida.operator.ALDOperator;
import de.unihalle.informatik.Alida.workflows.ALDWorkflow;
import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.Alida.annotations.Parameter.Direction;
import de.unihalle.informatik.Alida_xml.ALDXMLKeyValuePairType;
import de.unihalle.informatik.Alida_xml.ALDXMLObjectType;
import de.unihalle.informatik.Alida_xml.ALDXMLParametrizedType;
import de.unihalle.informatik.Alida_xml.ALDXMLWorkflowType;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Collection;
import java.util.LinkedList;
import java.util.HashMap;

import org.apache.xmlbeans.XmlDouble;
import org.apache.xmlbeans.XmlInt;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;

/**
 * DataIO provider for ALDWorkflow from xml using xmlbeans.
 * 
 * @author posch
 *
 */

// ensure higher priority then provider for ALDOperator
@ALDDataIOProvider(priority=2)
public class ALDWorkflowDataIOXmlbeans extends ALDStandardizedDataIOXmlbeans {

	/**
	 * debug messages
	 */
	private boolean debug = true;

	/**
	 * Interface method to announce class for which IO is provided for
	 * 
	 * @return  Collection of classes provided
	 */
	@Override
	public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();

		classes.add( ALDWorkflow.class);


		return classes;
	}

	// TODO: what should we do with workflowContext, resetDisconnectedInput
	@Override
	public Object readData(Field field, Class<?> cl, ALDXMLObjectType xmlObject, Object obj) 
			throws ALDDataIOProviderException, ALDDataIOManagerException {
		
		if ( debug ) {
			System.out.println( "ALDWorkflowDataIOXmlbeans::readData");
		}
		
		// allow additional fields when reading nested xml objects
		// WARNING: this is not thread save
		boolean saveAllowAdditionalFields = ALDDataIOManagerXmlbeans.getInstance().isAllowAdditionalFields();
		ALDDataIOManagerXmlbeans.getInstance().setAllowAdditionalFields(true); 
		
		
		// (optionally) create a workflow object
		ALDWorkflow workflow;
		if ( obj == null ) {
			try {
				workflow = new ALDWorkflow();
			} catch (ALDOperatorException e1) {
				throw new ALDDataIOProviderException( ALDDataIOProviderExceptionType.OBJECT_INSTANTIATION_ERROR,
						"ALDWorkflowDataIOXmlbeans::readData cannot instantiate class <ALDWorkflow>");
			}
		} else {
			try {
				workflow = (ALDWorkflow) obj;
			} catch (Exception e) {
				throw new ALDDataIOProviderException(ALDDataIOProviderExceptionType.OBJECT_TYPE_ERROR,
						"ALDWorkflowDataIOXmlbeans::readData obj has invalid class<" +
								obj.getClass().getName() + ">");
			}
		}
		
		
//		ALDXMLParametrizedType xmlParametrized = (ALDXMLParametrizedType) xmlObject;
//		for ( int i = 0 ; i < xmlParametrized.getPairsArray().length ; i++ )  {
//			String name = xmlParametrized.getPairsArray(i).getKey();
//			System.out.println("XX " + name);
//			ALDXMLObjectType elementXmlObject = xmlParametrized.getPairsArray(i).getValue();
//			if ( elementXmlObject != null) {
//
//				if ( name.equals("tester") ) {
//					Integer value = null;
//
//					try {
//						value = (Integer) ALDDataIOManagerXmlbeans.getInstance().readData( null, Integer.class,elementXmlObject);
//						if ( debug ) {
//							System.out.println(" ALDParametrizedClassDataIOXmlbeans::readData set <" +
//									name + "> to value <" + value + ">");
//						}
//					} catch (ALDDataIOManagerException e) {
//						throw new ALDDataIOManagerException( e.getType(), 
//								"ALDParametrizedClassDataIOXmlbeans::readData cannot read element <" +
//										name + "> \n    of class <" +
//										Integer.class + ">\n    from <" + 
//										elementXmlObject.toString() + ">" + 
//										"\n" + e.getCommentString());
//					} catch (ALDDataIOProviderException e) {
//						throw new ALDDataIOProviderException( e.getType(), 
//								"ALDParametrizedClassDataIOXmlbeans::readData cannot read element <" +
//										name + ">\n     of class <" +
//										Integer.class + ">\n     from <" + 
//										elementXmlObject.toString() + ">" + 
//										"\n" + e.getCommentString());
//					}
//				}
//
//			}
//		}

		// reset allowAdditionalFields in manager singleton
		ALDDataIOManagerXmlbeans.getInstance().setAllowAdditionalFields(saveAllowAdditionalFields); 

		return workflow;
	}

	@Override
	public ALDXMLObjectType writeData(Object obj) throws ALDDataIOProviderException, ALDDataIOManagerException  {
		
//		// first we write all fields annotated as @Parameter
//		ALDDataIOXmlbeans provider = (ALDDataIOXmlbeans) ALDDataIOManagerXmlbeans.getInstance().
//				getProvider(ALDParametrizedClassDummy.class, ALDDataIOXmlbeans.class);
//		ALDXMLParametrizedType aldXmlWorkflow =  (ALDXMLParametrizedType) provider.writeData(obj);
//		
//		Integer value = 123;
//		ALDXMLObjectType xmlValue = ALDDataIOManagerXmlbeans.getInstance().writeData(value);
//		ALDXMLKeyValuePairType keyValuePair = 
//				aldXmlWorkflow.addNewPairs();
//		keyValuePair.setKey("tester");
//		keyValuePair.setValue(xmlValue);
		
		Class<?> cl = obj.getClass();

		if ( obj instanceof ALDWorkflow ) {
			ALDXMLWorkflowType aldXmlWorkflow = ALDXMLWorkflowType.Factory.newInstance();
			aldXmlWorkflow.setClassName(obj.getClass().getName());
			
			ALDWorkflow workflow = (ALDWorkflow) obj;
			ALDParametrizedClassDataIOXmlbeans provider = new ALDParametrizedClassDataIOXmlbeans();
			ALDXMLParametrizedType xmlOperator = 
					(ALDXMLParametrizedType)provider.writeData(workflow);
			aldXmlWorkflow.setOperator(xmlOperator);
			aldXmlWorkflow.setResetDisconnectedInput(workflow.isResetDisconnectedInput());
				
			return aldXmlWorkflow;
		} else {
			throw new ALDDataIOProviderException(ALDDataIOProviderExceptionType.OBJECT_TYPE_ERROR,
					"ALDWorkflowDataIOXmlbeans::writeData invalid class<" +
					cl.getName() + ">");
		}
	}
}
