/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev$
 * $Date$
 * $Author$
 *
 */
package de.unihalle.informatik.Alida.demo;

import java.util.List;
import java.util.Set;

import javax.jws.soap.SOAPBinding.ParameterStyle;

import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDProcessingDAGException;
import de.unihalle.informatik.Alida.operator.ALDOperator;
import de.unihalle.informatik.Alida.annotations.Parameter;
import de.unihalle.informatik.Alida.annotations.ALDAOperator;

/**
 * Demo operator to summarize the cluster detecteded within experimental data.
 * 
 * @author posch
 */
@ALDAOperator(genericExecutionMode=ALDAOperator.ExecutionMode.ALL,
                level=ALDAOperator.Level.APPLICATION)
public class SummarizeClusters extends ALDOperator {

    /**
     * Input data
     */
    @Parameter(label= "Experimental data", required = true, 
          direction = Parameter.Direction.IN, description = "Experimental data to cluster")
    private ExperimentalData experiment = null;

	/**
     * Clusters represented as a list of clusters, where each cluster
     * is a set of experiment Ids, i.e. indices into to input experimental data.
     */
    @Parameter( label= "Clusters",  
          direction = Parameter.Direction.IN, description = "Clusters")
    private transient List<Set<Integer>> clusters= null;


    /**
     * Summary of experiment .
     */
    @Parameter(label= "Summary",
          direction = Parameter.Direction.OUT, description = "Summary of experimental data")
    private transient ExperimentalData summaryOfExperiment = null;

  /**
     * Default constructor.
     * @throws ALDOperatorException
     */
    public SummarizeClusters() throws ALDOperatorException {
    }

    /**
     * Constructor.
     * 
     * @param experiment    Experimental data
     * @throws ALDOperatorException
     */
    public SummarizeClusters(ExperimentalData _experiment) throws ALDOperatorException {
        this.experiment = _experiment;
    }

    @Override
    protected void operate()  throws ALDOperatorException,ALDProcessingDAGException {
        this.summaryOfExperiment = new ExperimentalData( this.experiment.getDescription() + " (Filtered)", 
        		experiment.getData(), false);

    }
 
    /** Get value of experiment.
      * Explanation: Experimental data to be normalized.
      * @return value of data
      */
    public ExperimentalData getExperiment(){
        return this.experiment;
    }

    /** Set value of data.
      * Explanation: Experimental data to be normalized.
      * @param value New value of data
      */
    public void setExperiment( ExperimentalData value){
        this.experiment = value;
    }

	/**
	 * @return the clusters
	 */
	public List<Set<Integer>> getClusters() {
		return clusters;
	}

	/**
	 * @param clusters the clusters to set
	 */
	public void setClusters(List<Set<Integer>> clusters) {
		this.clusters = clusters;
	}

	/**
	 * @return the summaryOfExperiment
	 */
	public ExperimentalData getSummaryOfExperiment() {
		return summaryOfExperiment;
	}
}

