/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

package de.unihalle.informatik.Alida.gui;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import de.unihalle.informatik.Alida.gui.ALDOperatorGUIExecutionProxy.WorkflowThreadStatus;
import de.unihalle.informatik.Alida.operator.ALDOperatorLocation;

/**
 * JUnit test class for {@link ALDOperatorGUIExecutionProxy}.
 * 
 * @author moeller
 */
public class TestALDOperatorGUIExecutionProxy {

	/**
	 * Object to be tested.
	 */
	private ALDOperatorGUIExecutionProxy guiProxy;
	
	/**
	 * Fixture.
	 */
	@Before
	public void initTestClass() { 
		ALDOperatorLocation location =
				ALDOperatorLocation.createClassLocation(
						"de.unihalle.informatik.Alida.operator.ALDFailingOperatorStub");
		this.guiProxy = new ALDOperatorGUIExecutionProxy(location);
	}
	
	/**
	 * Test if GUI proxy receives failure event on failing operator call and
	 * updates status accordingly.
	 */
	@Test
	public void testBehaviourOnFailingWorkflowExecution() {
		assertTrue("Workflow is not ready after initialization!",
			this.guiProxy.workflowStatus.equals(WorkflowThreadStatus.THREAD_READY));
		this.guiProxy.runWorkflow();
		// wait a certain amount of time to let all threads finish
		try {
			System.out.println("Going to sleep for 2.5 seconds...");
	    Thread.sleep(2500);
    } catch (InterruptedException e) {
	    e.printStackTrace();
    }
		assertTrue("Operator failed, GUI proxy should be ready again, but status "
				+ "is " + this.guiProxy.workflowStatus + "!",
			this.guiProxy.workflowStatus.equals(WorkflowThreadStatus.THREAD_READY));
	}
}